Package: rngtools
Authors@R: 
  c(person("Renaud", "Gaujoux", email = "renozao@protonmail.com", role = c("aut", "cre")),
    person("Max", "Kuhn", role = "ctb"))
Version: 1.5
License: GPL-3
Title: Utility Functions for Working with Random Number Generators
Description: Provides a set of functions for working with
    Random Number Generators (RNGs). In particular, a generic
    S4 framework is defined for getting/setting the current RNG, or RNG data
    that are embedded into objects for reproducibility.
    Notably, convenient default methods greatly facilitate the way current
    RNG settings can be changed.
URL: https://renozao.github.io/rngtools
BugReports: http://github.com/renozao/rngtools/issues
Encoding: UTF-8
Depends: R (>= 3.2.0), methods
Imports: digest, utils, stats, parallel
Suggests: covr, RUnit, testthat
RoxygenNote: 7.0.2
NeedsCompilation: no
Packaged: 2020-01-22 17:43:08 UTC; renaud
Author: Renaud Gaujoux [aut, cre],
  Max Kuhn [ctb]
Maintainer: Renaud Gaujoux <renozao@protonmail.com>
Repository: CRAN
Date/Publication: 2020-01-23 23:20:02 UTC
