# Generated by roxygen2: do not edit by hand

export(.getRNG)
export(.setRNG)
export(RNGdigest)
export(RNGinfo)
export(RNGrecovery)
export(RNGseed)
export(RNGseq)
export(RNGseq_seed)
export(RNGstr)
export(RNGtype)
export(checkRNG)
export(getRNG)
export(getRNG1)
export(hasRNG)
export(nextRNG)
export(rng.equal)
export(rng1.equal)
export(setRNG)
export(showRNG)
import(digest)
import(methods)
importFrom(parallel,nextRNGStream)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,tail)
