[![Build Status](https://travis-ci.org/renozao/rngtools.png?branch=master)](https://travis-ci.org/renozao/rngtools)
[![codecov](https://codecov.io/gh/renozao/rngtools/branch/master/graph/badge.svg)](https://codecov.io/gh/renozao/rngtools)

rngtools
========

R package - Utility functions for working with Random Number Generators

This package contains a set of functions for working with
Random Number Generators (RNGs). In particular, it defines a generic
S4 framework for getting/setting the current RNG, or RNG data
that are embedded into objects for reproducibility.

Notably, convenient default methods greatly facilitate the way current
RNG settings can be changed.
